DATE_FORMAT = "%d.%m.%Y"
ADDRESSBOOK_FILE = "addressbook.json"
NOTES_FILE = "notes.json"
WEEKDAYS = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday", "Next Monday"]
MONDAY_INDEX = 0
NEXT_MONDAY_INDEX = -1
PROMPT_STYLE = {
    # User input (default text).
    '': '#ffffff',
    # Prompt, approximation of Fore.LIGHTGREEN_EX
    'input_request': '#32c986',
}
PROMPT_MESSAGE = [
    ('class:input_request', 'Enter a command:'),
]
