from collections import UserDict

from contacts24.models.note import Note

# Temporary contract
class Notes(UserDict):
    def add_note(self, note: Note) -> None:
        # To complete by Sergii Shulga
        pass

def get_notes():
    pass


def create_note():
    pass


def update_note():
    pass


def delete_note():
    pass
